@@ -86,7 +86,7 @@ template<typename Str, typename Itr> class pkgCache::Iterator :
 	void ReMap(void const * const oldMap, void const * const newMap) {
 		if (Owner == 0 || S == 0)
 			return;
-		S += (Str const * const)(newMap) - (Str const * const)(oldMap);
+		S += static_cast<Str const *>(newMap) - static_cast<Str const *>(oldMap);
 	}
 
 	// Constructors - look out for the variable assigning
@@ -341,7 +341,7 @@ class pkgCache::DepIterator : public Iterator<Dependency, DepIterator> {
 		Iterator<Dependency, DepIterator>::ReMap(oldMap, newMap);
 		if (Owner == 0 || S == 0 || S2 == 0)
 			return;
-		S2 += (DependencyData const * const)(newMap) - (DependencyData const * const)(oldMap);
+		S2 += static_cast<DependencyData const *>(newMap) - static_cast<DependencyData const *>(oldMap);
 	}
 
 	//Nice printable representation
