@@ -13,7 +13,7 @@
 #include <apt-pkg/macros.h>
 
 #define INIT_FCS  0xffff
-unsigned short AddCRC16Byte(unsigned short fcs, unsigned char byte) APT_CONST;
+unsigned short AddCRC16Byte(unsigned short fcs, unsigned char byte) APT_PURE;
 unsigned short AddCRC16(unsigned short fcs, void const *buf,
 			unsigned long long len) APT_PURE;
 
