@@ -63,19 +63,18 @@
    ##################################################################### */
 									/*}}}*/
 // Include Files							/*{{{*/
-#include<config.h>
+#include <config.h>
 
-#include <apt-pkg/orderlist.h>
+#include <apt-pkg/configuration.h>
 #include <apt-pkg/depcache.h>
 #include <apt-pkg/error.h>
-#include <apt-pkg/configuration.h>
-#include <apt-pkg/cacheiterators.h>
+#include <apt-pkg/orderlist.h>
 #include <apt-pkg/pkgcache.h>
 
-#include <stdlib.h>
-#include <string.h>
 #include <algorithm>
 #include <iostream>
+#include <stdlib.h>
+#include <string.h>
 									/*}}}*/
 
 using namespace std;
@@ -93,7 +92,7 @@ pkgOrderList::pkgOrderList(pkgDepCache *pCache) : d(NULL), Cache(*pCache),
 
    /* Construct the arrays, egcs 1.0.1 bug requires the package count
       hack */
-   unsigned long Size = Cache.Head().PackageCount;
+   auto const Size = Cache.Head().PackageCount;
    Flags = new unsigned short[Size];
    End = List = new Package *[Size];
    memset(Flags,0,sizeof(*Flags)*Size);
@@ -1056,8 +1055,8 @@ bool pkgOrderList::AddLoop(DepIterator D)
 /* */
 void pkgOrderList::WipeFlags(unsigned long F)
 {
-   unsigned long Size = Cache.Head().PackageCount;
-   for (unsigned long I = 0; I != Size; I++)
+   auto Size = Cache.Head().PackageCount;
+   for (decltype(Size) I = 0; I != Size; ++I)
       Flags[I] &= ~F;
 }
 									/*}}}*/
