@@ -1,7 +1,7 @@
 #include <config.h>
 
-#include <apt-pkg/sourcelist.h>
 #include <apt-pkg/fileutl.h>
+#include <apt-pkg/sourcelist.h>
 
 #include <string>
 #include <stdlib.h>
@@ -12,32 +12,28 @@
 
 #include "file-helpers.h"
 
-class SourceList : public pkgSourceList {
-   public:
-      using pkgSourceList::ParseFileDeb822;
-};
-
 TEST(SourceListTest,ParseFileDeb822)
 {
    FileFd fd;
-   char * tempfile;
-   createTemporaryFile("parsefiledeb822", fd, &tempfile,
+   std::string tempfile;
+   createTemporaryFile("parsefiledeb822.XXXXXX.sources", fd, &tempfile,
       "Types: deb\n"
       "URIs: http://ftp.debian.org/debian\n"
       "Suites: stable\n"
-      "Sections: main\n"
+      "Components: main\n"
       "Description: short\n"
       " long description that can be very long\n"
       "\n"
       "Types: deb\n"
       "URIs: http://ftp.debian.org/debian\n"
       "Suites: unstable\n"
-      "Sections: main non-free\n");
+      "Components: main non-free\n");
    fd.Close();
 
-   SourceList sources;
-   EXPECT_EQ(2, sources.ParseFileDeb822(tempfile));
+   pkgSourceList sources;
+   EXPECT_EQ(true, sources.Read(tempfile));
    EXPECT_EQ(2, sources.size());
 
-   unlink(tempfile);
+   if (tempfile.empty() == false)
+      unlink(tempfile.c_str());
 }
